/*
*  MPInfo utility
*  
*  source/mp3lib.c
*  snapshot 20040218
*  
*  (c) 13.8.2002 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/


#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "common.h"
#include "mp3lib.h"


#define HEADSIZE 128
#define T1SPCHAR '\0'


int gettag1(const char *fname, struct tag1_ent *tag)
{
        int ret;

        if((ret = get_rough_tag1(fname, tag)) == 0){
                strsanity(tag->name);
                strsanity(tag->artist);
                strsanity(tag->album);
                strsanity(tag->year);
                strsanity(tag->comment);
        } else {
                tag->name[0] = '\0';
                tag->artist[0] = '\0';
                tag->album[0] = '\0';
                tag->year[0] = '\0';
                tag->comment[0] = '\0';

        }

        return ret;
}


int get_rough_tag1(const char *fname, struct tag1_ent *tag)
{
        FILE *f;
        struct stat attrs;
        int i;


        if(stat(fname, &attrs))
            return -1;

        if(attrs.st_size < HEADSIZE)
            return 1;

        if((f = fopen(fname, "rb")) == NULL)    
            return -1;
  
        fseek(f, attrs.st_size - HEADSIZE, SEEK_SET);
        i = fread(tag->tag, sizeof(char), T1TAGLEN - 1, f);
        tag->tag[i] = '\0';
        if(strcmp(tag->tag, "TAG")){
            fclose(f);
            return 1;
        }

        i = fread(tag->name, sizeof(char), T1NAMELEN - 1, f);
        tag->name[i] = '\0';

        i = fread(tag->artist, sizeof(char), T1ARTISTLEN - 1, f);
        tag->artist[i] = '\0';

        i = fread(tag->album, sizeof(char), T1ALBUMLEN - 1, f);
        tag->album[i] = '\0';

        i = fread(tag->year, sizeof(char), T1YEARLEN - 1, f);
        tag->year[i] = '\0';

        i = fread(tag->comment, sizeof(char), T1COMMENTLEN - 1, f);
        tag->comment[i] = '\0';

        i = fread(&tag->genre, sizeof(char), 1, f);
        if(!i) tag->genre = 0;

        fclose(f);
        return 0;
}


int puttag1(const char *fname, struct tag1_ent *tag)
{
        FILE *f;
        struct stat attrs;
        int i;


        if(stat(fname, &attrs))
            return -1;
        
        if((f = fopen(fname, "rb+")) == NULL)    
            return -1;
  
        if(attrs.st_size > HEADSIZE)  
            fseek(f, attrs.st_size - HEADSIZE, SEEK_SET);
        i = fread(tag->tag,sizeof(char), T1TAGLEN - 1, f);
        tag->tag[i] = '\0';

        if(strcmp(tag->tag, "TAG")){
                fseek(f, 0, SEEK_END);
                strcpy(tag->tag, "TAG");
                fwrite(tag->tag, sizeof(char), T1TAGLEN - 1, f);
        }

        fseek(f, 0, SEEK_CUR);
        trim_or_fill(tag->name, T1NAMELEN - 1, T1SPCHAR);
        fwrite(tag->name, sizeof(char), T1NAMELEN - 1, f);
        trim_or_fill(tag->artist, T1ARTISTLEN - 1, T1SPCHAR);
        fwrite(tag->artist, sizeof(char), T1ARTISTLEN - 1, f);
        trim_or_fill(tag->album, T1ALBUMLEN - 1, T1SPCHAR);
        fwrite(tag->album, sizeof(char), T1ALBUMLEN - 1, f);
        trim_or_fill(tag->year, T1YEARLEN - 1, T1SPCHAR);
        fwrite(tag->year, sizeof(char), T1YEARLEN - 1, f);
        trim_or_fill(tag->comment, T1COMMENTLEN - 1, T1SPCHAR);
        fwrite(tag->comment, sizeof(char), T1COMMENTLEN - 1, f);
        fwrite(&tag->genre, sizeof(char), 1, f);

        fclose(f);
        return 0;
}
