/*
*  MPInfo utility
*  
*  source/mpinfo.c
*  snapshot 20040218
*  
*  (c) 13.8.2002 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  License GNU/GPL2  
*/

#define VERSION "MPInfo snapshot 20040218 cz"


#include <stdio.h>
#include <string.h>
#include "common.h"
#include "mp3lib.h"


/* implicitni maska */
#define MASK "ianygcp"
#define MAXBUFF 512


struct modify_tag {
        char *album;
        char *comment;
        char *interpret;
        char *name;
        char *year;
};


int run(char *mp3name, char *mask, int format, int modify, struct modify_tag *ms);

/* vypise polozky tagu pod sebe */
void tag1_print(struct tag1_ent *tag, char *name, char *mask);

/* vypise polozky tagu do jednoho radku */
void tag1_printl(struct tag1_ent *tag, char *name, char *mask);

/* vypise polozky tagu do radku tabulky html */
void tag1_printt(struct tag1_ent *tag, char *name, char *mask);

/* vytiskne help, parametr je jmeno programu */
void printhelp(char *program);


int main(int argc, char *argv[])
{
        struct modify_tag ms;
        int count,
            format = 0,
            modify = 0;
        char *argument = NULL,
             *mp3name = NULL,
             *mask = NULL,
             line[MAXBUFF];


        ms.album = NULL;
        ms.comment = NULL;
        ms.interpret = NULL;
        ms.name = NULL;
        ms.year = NULL;
  
        count = argc - 1;
        while(count > 0){
                if(argv[count][0] == '-'){
                        switch(argv[count][1]){
                                case 'a': 
                                        ms.album = argument;
                                        modify = 1;
                                        break;
                                case 'c': 
                                        ms.comment = argument;
                                        modify = 1;
                                        break;
                                case 'f': 
                                        mp3name = argument;
                                        break;
                                case 'h': 
                                        printhelp(argv[0]);
                                        return 0;
                                case 'i': 
                                        ms.interpret = argument;
                                        modify = 1;
                                        break;
                                case 'l':
                                        format = 1;
                                        mask = argument;
                                        break;
                                case 'n': 
                                        ms.name = argument;
                                        modify = 1;
                                        break;
                                case 't': 
                                        format = 2;
                                        mask = argument;
                                        break;
                                case 'y': 
                                        ms.year = argument;
                                        modify = 1;
                                        break;
                        }
                        argument = NULL;
                } else {
                        argument = argv[count];                        
                }
                count--;
        }

        if (mp3name == NULL) {

                while (fgets(line, MAXBUFF, stdin) != NULL){
                        strip_sp(line);
                        run(line, mask, format, modify, &ms);
                }

                return 0;
        } else
                return run(mp3name, mask, format, modify, &ms);
}


int run(char *mp3name, char *mask, int format, int modify, struct modify_tag *ms)
{
        struct tag1_ent mpinfo;
        int tagstatus;


        if ((tagstatus = gettag1(mp3name, &mpinfo)) == -1){
                printf("Soubor %s nelze orevrit pro cteni\n", mp3name);
                return -1;
        }

        if (modify){
                if (tagstatus){
                        strcpy(mpinfo.album,"");
                        strcpy(mpinfo.comment,"");
                        strcpy(mpinfo.artist,"");
                        strcpy(mpinfo.name,"");
                        strcpy(mpinfo.year,"");
                        mpinfo.genre=(unsigned char) 0xFF;
                }

                if (ms->album != NULL)
                        strncpy(mpinfo.album, ms->album, 30);
                if (ms->comment != NULL)
                        strncpy(mpinfo.comment, ms->comment, 30);
                if (ms->interpret != NULL)
                        strncpy(mpinfo.artist, ms->interpret, 30);
                if (ms->name != NULL)
                        strncpy(mpinfo.name, ms->name, 30);
                if (ms->year != NULL)
                        strncpy(mpinfo.year, ms->year, 4);

                if (puttag1(mp3name, &mpinfo)){
                        printf("Soubor %s nelze modifikovat\n", mp3name);
                        return -1;
                }
                tagstatus = gettag1(mp3name, &mpinfo);
        }

        if(!tagstatus){
                switch (format){    	        
                        case 1:
                                tag1_printl(&mpinfo, mp3name, mask);
                                break;
                        case 2:
                                tag1_printt(&mpinfo, mp3name, mask);
                                break;
                        default:
                                tag1_print(&mpinfo, mp3name, NULL);
                }
        }

        return 0;
}


void tag1_print(struct tag1_ent *tag, char *name, char *mask)
{
        unsigned n = 0;


        if(mask == NULL)
                mask = MASK;

        while(mask[n] != '\0'){
                switch(mask[n]){
                        case 'i' :
                                printf("Interpret: %s\n", tag->artist);
                                break;
                        case 'a' :
                                printf("Album:     %s\n", tag->album);
                                break;
                        case 'n' :
                                printf("Name:      %s\n", tag->name);
                                break;
                        case 'y' :
                                printf("Year:      %s\n", tag->year);
                                break;
                        case 'g' :
                                printf("Genre:     %d\n", tag->genre);
                                break;
                        case 'c' :
                                printf("Comment:   %s\n", tag->comment);
                                break;
                        case 'p' :
                                printf("Path:      %s\n", name);
                                break;
                }
                n++;
        }

        return;
}


void tag1_printl(struct tag1_ent *tag, char *name, char *mask)
{
        unsigned n = 0;


        if(mask == NULL)
                mask = MASK;

        while(mask[n] != '\0'){
                switch(mask[n]){
                        case 'i' :
                                zarovnat(tag->artist,30,' ');
                                printf("%s ", tag->artist);
                                break;
                        case 'a' :
                                zarovnat(tag->album,30,' ');
                                printf("%s ", tag->album);
                                break;
                        case 'n' :
                                zarovnat(tag->name,30,' ');
                                printf("%s ", tag->name);
                                break;
                        case 'y' :
                                zarovnat(tag->year,4,' ');
                                printf("%s ", tag->year);
                                break;
                        case 'g' :
                                printf("%3.0d ", tag->genre);
                                break;
                        case 'c' :
                                zarovnat(tag->comment,30,' ');
                                printf("%s ", tag->comment);
                                break;
                        case 'p' :
                                printf("%s ", name);
                                break;
                }
                n++;
        }

        puts("");
        return;
}


void tag1_printt(struct tag1_ent *tag, char *name, char *mask)
{
        unsigned n = 0;

        if(mask == NULL)
                mask = MASK;

        printf("<tr>");
        while(mask[n] != '\0'){
                switch(mask[n]){
                        case 'i' :
                                printf("<td>%s</td>", tag->artist);
                                break;
                        case 'a' :
                                printf("<td>%s</td>", tag->album);
                                break;
                        case 'n' :
                                printf("<td>%s</td>", tag->name);
                                break;
                        case 'N' :
                                printf("<td><a href=\"file:///%s\">%s</a></td>"
                                        , name, tag->name);
                                break;
                        case 'y' :
                                printf("<td>%s</td>", tag->year);
                                break;
                        case 'g' :
                                printf("<td>%d</td>", tag->genre);
                                break;
                        case 'c' :
                                printf("<td>%s</td>", tag->comment);
                                break;
                        case 'p' :
                                printf("<td>%s</td>", name);
                                break;
                }
                n++;
        }        
        puts("</tr>");
        return;
}


void printhelp(char *program)
{
        printf("%s - program pro zjistovani informaci z mp3 souboru\n", VERSION);
        printf("(c)2002 Jaroslav Beran - berk@volny.cz http://www.volny.cz/berk\n");
        printf("Licence GNU/GPL2 (http://www.gnu.cz)\n\n");
        printf("\n");
        printf("Pouziti:\n");
        printf("%s [volby] soubor.mp3\n", program);
        printf("\n");
        printf("Volby:\n");
        printf("-a album       zapise/zmeni jmeno alba, maximalne 30 zaku\n");
        printf("-c comment     zapise/zmeni jmeno komentar, maximalne 30 zaku\n");
        printf("-f filename    jmeno mp3 souboru\n");
        printf("-h             help\n");
        printf("-i interpret   zapise/zmeni jmeno interpreta, maximalne 30 zaku\n");
        printf("-l [poradi]    vypsat jako radek tabulky\n");
        printf("-n name        zapise/zmeni jmeno skladby, maximalne 30 zaku\n");
        printf("-t [poradi]    vypsat jako radek html tabulky\n");
        printf("-y year        zapise/zmeni rok, maximalne 4 zaky\n");
        printf("\n");
        printf("--vice--\n");
        getchar();
        printf("Poradi:\n");
        printf("n   name\n");
        printf("N   name jak hyperlink (pouze u volby -t)\n");
        printf("a   album\n");
        printf("i   interpret\n");
        printf("y   year\n");
        printf("c   comment\n");
        printf("g   genre\n");
        printf("p   path\n");

        return;
}
